import os

import bpy

from .main_exporter import MainExporter
from .submodel_maker import SubmodelMaker
from .t3d_file_writer import T3DFileWriter
from .t3d_submodels.t3d_mesh import T3DMesh
from .t3d_submodels.t3d_stars import T3DStars
from .t3d_submodels.getters.map_property_getter import MapPropertyGetter
from .t3d_submodels.getters.stars_getter import StarsGetter
from .t3d_submodels.getters.triangles_getter import TrianglesGetter


class ExporterBuilder:
    def get_exporter(self) -> MainExporter:
        submodel_maker = SubmodelMaker()
        t3d_file_writer = T3DFileWriter()

        bl_prefs = bpy.context.user_preferences

        base_textures_path = os.path.join(
            bl_prefs.addons["maszyna_t3d_export"].preferences.root_path,
            "textures"
        )
        map_property_getter = MapPropertyGetter(base_textures_path)
        triangles_getter = TrianglesGetter()
        T3DMesh.map_property_getter = map_property_getter
        T3DMesh.triangles_getter = triangles_getter

        stars_getter = StarsGetter()
        T3DStars.stars_getter = stars_getter

        return MainExporter(submodel_maker, t3d_file_writer)
