from .t3d_submodels.t3d_submodel import T3DSubmodel
from .writer_states.freespotlight_writer_state import FreespotligtWriterState
from .writer_states.mesh_writer_state import MeshWriterState
from .writer_states.stars_writer_state import StarsWriterState


class T3DFileWriter:
    def __init__(self):
        self.state = None
        self.states = {
            "Mesh": MeshWriterState(),
            "FreeSpotLight": FreespotligtWriterState(),
            "Stars": StarsWriterState()
        }

    def write_submodel(self, submodel: T3DSubmodel, output_file):
        self.output_file = output_file
        self._switch_state(submodel.get_type())

        self._write_separator()
        self._write_first_group_of_common_properties(submodel)
        self.state.write_specific_properties(submodel, self.output_file)
        self._write_last_group_of_common_properties(submodel)
        self.state.write_verts_groups(submodel, self.output_file)

    def _switch_state(self, submodel_type: str):
        self.state = self.states[submodel_type]

    def _write_separator(self, length: int=81):
        self.output_file.write(
            "//{}\n".format("".join(["-"] * length)))

    def _write_first_group_of_common_properties(self, submodel: T3DSubmodel):
        self.output_file.writelines("".join((
            "Parent: {}\n".format(submodel.get_parent()),
            "Type: {}\n".format(submodel.get_type()),
            "Name: {}\n".format(submodel.get_name()),
            "Anim: {}\n".format(submodel.get_anim())
        )))

    def _write_last_group_of_common_properties(self, submodel: T3DSubmodel):
        self.output_file.writelines("".join((
            "MaxDistance: {}\n".format(submodel.get_max_distance()),
            "MinDistance: {}\n".format(submodel.get_min_distance()),
            "Transform:\n{}\n".format(submodel.get_transform()),
        )))
