import os

import bpy


class MapPropertyGetter:
    def __init__(self, base_textures_path: str):
        self.base_textures_path = base_textures_path

    def get_map(self, material: bpy.types.Material,
                replacableskin_id: str) -> str:
        if replacableskin_id == "none":
            return self._get_map_from_active_texture(material.active_texture)
        else:
            return replacableskin_id

    def _get_map_from_active_texture(self, tex: bpy.types.ImageTexture) -> str:
        try:
            abspath = tex.image.filepath_from_user()
        except AttributeError:
            return "none"
        else:
            return self._get_relative_path(abspath)

    def _get_relative_path(self, abspath: str) -> str:
        if "dynamic" in abspath:
            path = os.path.basename(abspath)
        elif "textures" in abspath:
            path = os.path.relpath(abspath, self.base_textures_path)
        return os.sep + os.path.splitext(path)[0]
