from typing import Iterator, Iterable
import math

import bpy

from maszyna_t3d_export.src.utils import format_float


class StarsGetter:
    def get_stars(self, stars: Iterable) -> Iterator[str]:
        return self._stars_generator(stars)

    def _stars_generator(self, stars: Iterable) -> Iterator[str]:
        verts_group_number = math.ceil(len(stars) / 3)
        for verts_group_counter in range(verts_group_number):

            slice_start = verts_group_counter * 3
            slice_end = slice_start + 3
            stars_slice = stars[slice_start:slice_end]
            stars_definitions = [
                self._get_star_definition(star) for star in stars_slice
            ]

            yield "1\n" + "\n".join(stars_definitions)

    def _get_star_definition(self, star: bpy.types.PointLamp) -> str:
        return " ".join((
            " ".join([format_float(number) for number in star.location]),
            self._compress_color(star.data.color),
            "0"
        ))

    def _compress_color(self, color: Iterable) -> str:
        return str((int(color[2] * 255) << 16)
                   | (int(color[1] * 255) << 8)
                   | int(color[0] * 255))
