from typing import Iterator, List

import bpy

from maszyna_t3d_export.src.utils import format_float


class TrianglesGetter:
    def get_triangles(self, mesh: bpy.types.Mesh) -> Iterator[str]:
        self.mesh = mesh
        self.smooth_groups = self.mesh.calc_smooth_groups(True)
        self.active_uv_layer = mesh.uv_layers.active

        return self._generate_triangles()

    def _generate_triangles(self) -> Iterator[str]:
        for triangle in self.mesh.polygons:
            yield "\n".join((
                self._get_smooth_mask(triangle),
                self._get_coords(triangle)
            ))

    def _get_smooth_mask(self, triangle: bpy.types.MeshPolygon) -> str:
        if triangle.use_smooth:
            return str(self.smooth_groups[0][triangle.index])
        else:
            return "0"

    def _get_coords(self, triangle: bpy.types.Mesh) -> str:
        coords = []
        for local_vert_idx, global_vert_idx in enumerate(triangle.vertices):

            xyz = " ".join([
                format_float(num)
                for num in self.mesh.vertices[global_vert_idx].co
            ])

            if self.active_uv_layer is None:
                uv = "0 0"
            else:
                mesh_loop_index = triangle.loop_indices[local_vert_idx]
                uv = " ".join([
                    format_float(num)
                    for num in self.active_uv_layer.data[mesh_loop_index].uv
                ])

            coords.append(xyz + " " + uv)

        return "\n".join(coords)
