import math

import bpy

from .t3d_submodel import T3DSubmodel
from maszyna_t3d_export.src.utils import format_float


class T3DFreeSpotLight(T3DSubmodel):
    _submodel_typename = "FreeSpotLight"

    def __init__(self, obj: bpy.types.Object):
        super().__init__(obj)
        self.obj = obj

    def get_diffuse(self) -> str:
        diffuse = [str(int(c * 255)) for c in self.obj.data.color]
        return " ".join(diffuse)

    def get_selfillum(self) -> str:
        return "true"

    def get_near_atten_start(self) -> str:
        return "0.0"

    def get_near_atten_end(self) -> str:
        return "40.0"

    def get_use_near_atten(self) -> str:
        return "false"

    def get_far_atten_decay_type(self) -> str:
        return ("1" if self.obj.data.falloff_type == 'INVERSE_LINEAR'
                else "2" if self.obj.data.falloff_type == 'INVERSE_SQUARE'
                else "0")

    def get_far_decay_radius(self) -> str:
        return format_float(self.obj.data.distance)

    def get_falloff_angle(self) -> str:
        falloff_angle = math.degrees(
            self.obj.data.spot_size * (self.obj.data.spot_blend + 1))
        return format_float(falloff_angle)

    def get_hotspot_angle(self) -> str:
        hotspot_angle = math.degrees(self.obj.data.spot_size)
        return format_float(hotspot_angle)
