from typing import Iterator

import bpy

from .t3d_submodel import T3DSubmodel


class T3DMesh(T3DSubmodel):
    triangles_getter = None
    map_property_getter = None
    _submodel_typename = "Mesh"

    def __init__(self, obj: bpy.types.Object):
        super().__init__(obj)
        self.obj = obj

        triangulated_obj = self.obj.copy()
        triangulated_obj.modifiers.new("T3DTri", type='TRIANGULATE')
        self._triangulated_mesh = triangulated_obj.to_mesh(
            bpy.context.scene, True, 'PREVIEW')

    def get_ambient(self) -> str:
        if self.obj.active_material:
            ambient = [str(int(self.obj.active_material.ambient * 255))] * 3
            return " ".join(ambient)
        else:
            return "255 255 255"

    def get_diffuse(self) -> str:
        if self.obj.active_material:
            diffuse = [
                str(int(c * 255))
                for c in self.obj.active_material.diffuse_color
            ]
            return " ".join(diffuse)
        else:
            return "255 255 255"

    def get_specular(self) -> str:
        if self.obj.active_material:
            specular = [
                str(int(c * 255))
                for c in self.obj.active_material.specular_color
            ]
            return " ".join(specular)
        else:
            return "255 255 255"

    def get_wire(self) -> str:
        return "false"

    def get_wire_size(self) -> str:
        return "1.0"

    def get_opacity(self) -> str:
        return "0" if self.obj.active_material.use_transparency else "1"

    def get_map(self) -> str:
        return self.map_property_getter.get_map(
            self.obj.active_material,
            self.obj.t3d_properties.replacableskin_id)

    def get_numverts(self) -> str:
        return str(len(self._triangulated_mesh.polygons) * 3)

    def get_verts_groups(self) -> Iterator[str]:
        return self.triangles_getter.get_triangles(self._triangulated_mesh)
