import bpy

from maszyna_t3d_export.src.utils import format_float


class T3DSubmodel:
    _submodel_typename = ""

    def __init__(self, obj: bpy.types.Object):
        self.obj = obj

    def get_parent(self) -> str:
        return self.obj.parent.name if self.obj.parent is not None else "none"

    def get_name(self) -> str:
        return self.obj.name

    def get_type(self) -> str:
        return self._submodel_typename

    def get_anim(self) -> str:
        return self.obj.t3d_properties.anim

    def get_diffuse(self) -> str:
        return "255 255 255"

    def get_selfillum(self) -> str:
        if self.obj.t3d_properties.selfillum_false:
            return "false"
        elif self.obj.t3d_properties.selfillum_true:
            return "true"
        else:
            return format_float(self.obj.t3d_properties.selfillum)

    def get_max_distance(self) -> str:
        max_distance = self.obj.t3d_properties.max_distance
        if max_distance < 0:
            max_distance = "-1"
        return str(max_distance)

    def get_min_distance(self) -> str:
        return str(self.obj.t3d_properties.min_distance)

    def get_transform(self) -> str:
        matrix = (
            self.obj.matrix_basis * self.obj.matrix_parent_inverse
        ).transposed()

        return "\t" + "\n\t".join([
            " ".join([format_float(number) for number in row])
            for row in matrix
        ])
