bl_info = {
    "name": "MaSzyna T3D modeling tools",
    "description": "Addon for working with T3D models",
    "author": "Krzysztof Przybyla (krzysiuup)",
    "version": (1, 0, 0),
    "blender": (2, 78, 0),
    "location": "Properties > T3D",
    "wiki_url": "https://wiki.eu07.pl/index.php/Blender_-_modelowanie",
    "support": "OFFICIAL",
    "category": "User Interface",
}

import bpy
from bpy.props import (
    StringProperty,
    BoolProperty,
    EnumProperty,
    PointerProperty,
    IntProperty,
    FloatProperty
)


class T3DProperties(bpy.types.PropertyGroup):
    anim = EnumProperty(
        name="Animation",
        default="false",
        items=[
            ("ik22", "ik22", "IK: kierunkowany"),
            ("ik21", "ik21", "IK: kierunkowany"),
            ("ik11", "ik11", "IK: kierunkowany"),
            ("ik", "ik", "IK: zadający"),
            ("hours24_jump", "hours24_jump", "godziny z przeskokiem co 15°"),
            ("hours24", "hours24", "godziny płynnie (24h/360°)"),
            ("hours_jump", "hours_jump", "godziny z przeskokiem co 30°"),
            ("hours", "hours", "godziny płynnie (12h/360°)"),
            ("minutes_jump", "minutes_jump", "minuty z przeskokiem co 6°"),
            ("minutes", "minutes", "minuty płynnie (1h/360°, pełny obrót w godzinę)"),
            ("seconds_jump", "seconds_jump", "sekundy z przeskokiem co 6°"),
            ("seconds", "seconds", "sekundy płynnie (60s/360°, pełny obrót w minutę)"),
            ("digiclk", "digiclk", "animacja zegara cyfrowego"),
            ("billboard", "billboard", "obracanie w kierunku kamery"),
            ("wind", "wind", "kołysanie się obiektu pod wpływem wiatru"),
            ("sky", "sky", "animacja dla nocnego nieba"),
            ("true", "true", "umożliwia animowanie obiektu za pomocą eventów"),
            ("false", "false", "brak animacji")
        ]
    )
    max_distance = FloatProperty(
        name="Max Distance",
        description="Maksymalna odległość z jakiej obiekt jest widoczny",
        min=-1,
        default=-1,
        step=0.1
    )
    min_distance = IntProperty(
        name="Min Distance",
        description="Minimalna odległość z jakiej obiekt jest widoczny",
        min=0
    )
    selfillum = FloatProperty(
        name="SelfIllum",
        description="Zapal światła przy wybranym poziomie oświetlenia",
        default=0,
        min=0,
        max=1.25,
        step=0.01
    )
    selfillum_true = BoolProperty(
        name="True",
        description="Emituj światło przez cały czas, niezależnie od poziomu oświetlenia scenerii",
        default=False
    )
    selfillum_false = BoolProperty(
        name="False",
        description="Nigdy nie emituj światła",
        default=True
    )
    replacableskin_id = EnumProperty(
        name="Replacableskin ID",
        default="none",
        items=[
            ("none", "None", "brak tekstury wymiennej"),
            ("-1", "-1", ""),
            ("-2", "-2", ""),
            ("-3", "-3", ""),
            ("-4", "-4", "")
        ]
    )


class T3DPropertiesPanel(bpy.types.Panel):
    bl_idname = "OBJECT_PT_t3d_mesh"
    bl_label = "T3D Properties"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_context = 'object'

    @classmethod
    def poll(self, context):
        return context.object

    def draw(self, context):
        obj = context.object
        layout = self.layout

        box_distances = layout.box()
        row_max_dist = box_distances.row()
        row_min_dist = box_distances.row()
        row_max_dist.prop(obj.t3d_properties, "max_distance")
        row_min_dist.prop(obj.t3d_properties, "min_distance")

        box_sf_properties = layout.box()
        row_sf = box_sf_properties.row()
        row_sf_flags = box_sf_properties.row()
        col_sf_true = row_sf_flags.column()
        col_sf_false = row_sf_flags.column()

        row_sf.prop(obj.t3d_properties, "selfillum")
        col_sf_true.prop(obj.t3d_properties, "selfillum_true")
        col_sf_false.prop(obj.t3d_properties, "selfillum_false")
        layout.prop(obj.t3d_properties, "anim")
        row_replacableskin = layout.row()
        row_replacableskin.prop(obj.t3d_properties, "replacableskin_id")

        if obj.t3d_properties.selfillum_true:
            col_sf_false.enabled = False
            row_sf.enabled = False
        elif obj.t3d_properties.selfillum_false:
            col_sf_true.enabled = False
            row_sf.enabled = False


def register():
    bpy.utils.register_module(__name__)
    bpy.types.Object.t3d_properties = PointerProperty(type=T3DProperties)


def unregister():
    bpy.utils.unregister_module(__name__)
    del bpy.types.Object.t3d_properties


if __name__ == '__main__':
    register()
